library ieee;
use ieee.std_logic_1164.all;
use ieee.std_logic_arith.all;
use ieee.std_logic_unsigned.all;
use work.math_pack.all;
library unisim;
use UNISIM.vcomponents.all;

entity top_level is
    generic(
        -- rychlost v ms
        SPEED : integer := 1000       
    );
    port(
        -- hodiny
        CLK_100MHZ           : in std_logic;    
        -- reset
        GPIO_SW_C            : in std_logic;
        -- pauza/start
        GPIO_SW_N            : in std_logic;

        GPIO_LED_0           : out std_logic; 
        GPIO_LED_1           : out std_logic;
        GPIO_LED_2           : out std_logic;
        GPIO_LED_3           : out std_logic;
        GPIO_LED_4           : out std_logic;
        GPIO_LED_5           : out std_logic;
        GPIO_LED_6           : out std_logic;  
        GPIO_LED_7           : out std_logic;
    
        HDR1_56              : out std_logic; -- locked
        HDR1_60              : out std_logic; -- clk_sig
        HDR1_64              : out std_logic  -- 100MHz
    ); 
end top_level;



architecture top_level_structural of top_level is

    component counter is
    generic( 
        DATA_WIDTH        : integer
    );
    port( 
        CLK      : in std_logic;
        RST      : in std_logic;
        CLR      : in std_logic;
        EN       : in std_logic;
        DATA     : out std_logic_vector(DATA_WIDTH-1 downto 0)
    ); 
    end component;

    component simple_counter is
    generic( 
        DATA_WIDTH        : integer
    );
    port( 
        CLK      : in std_logic;
        RST      : in std_logic;
        CLR      : in std_logic;
        EN       : in std_logic;
        DATA     : out std_logic_vector(DATA_WIDTH-1 downto 0)
    ); 
    end component;

    component switch is
    generic (
        FREQ_CLK         : positive;          -- frekvence hodin
        LENGTH           : positive           -- delka stisku v ms
    );
    port (
        CLK              : in std_logic; 
        RST              : in std_logic;   
        SW               : in std_logic;
        PRESSED          : out std_logic;
        BUSY             : out std_logic
    ); 
    end component;

    component chipscope_icon is
    port (
        CONTROL0 : inout STD_LOGIC_VECTOR ( 35 downto 0 ); 
        CONTROL1 : inout STD_LOGIC_VECTOR ( 35 downto 0 ) 
    );
    end component;

    component chipscope_ila is
    port (
        CLK : in STD_LOGIC := 'X'; 
        CONTROL : inout STD_LOGIC_VECTOR ( 35 downto 0 ); 
        TRIG0 : in STD_LOGIC_VECTOR ( 7 downto 0 ); 
        DATA : in STD_LOGIC_VECTOR ( 95 downto 0 ) 
    );
    end component;


    component chipscope_ila1 is
    port (
        CLK: in std_logic;
        CONTROL : inout std_logic_vector(35 downto 0);
        TRIG0 : in std_logic_vector(7 downto 0);
        DATA : in std_logic_vector(15 downto 0)  
    );
    end component;

    type states is (
        INIT,
        --IDLE,
        COUNT,
        PAUSE
    );

    signal state : states := INIT;
    signal nextstate : states := INIT;

    signal clkin_ibufg,clkfx_buf,clk0_buf,clkfb_in : std_logic;

    signal clk_sig : std_logic;              -- period 10 ns
    signal rst_sig : std_logic;
   
    signal counter_data_sig : std_logic_vector(7 downto 0);
    signal counter_en_sig, counter_clr_sig : std_logic;

    signal delay_counter_data_sig : std_logic_vector(log2(SPEED*100000)-1 downto 0);
    signal delay_counter_en_sig, delay_counter_clr_sig : std_logic;
    
    signal delay_finished_sig, pause_sig : std_logic;

    signal locked_sig : std_logic;
     
    signal chipscope_data_sig : std_logic_vector(95 downto 0);
    signal chipscope_trigger_sig : std_logic_vector(7 downto 0);
    signal chip_controls, chip1_controls :  std_logic_vector(35 downto 0);

    attribute equivalent_register_removal: string;   
    attribute equivalent_register_removal of counter_data_sig : signal is "no";
    attribute equivalent_register_removal of delay_counter_data_sig : signal is "no";
    attribute keep:string;
    attribute keep of counter_data_sig : signal is "true";
    attribute keep of delay_counter_data_sig : signal is "true";

begin

    DCM_BASE_inst : DCM_BASE
    generic map (
        CLKDV_DIVIDE => 2.0,--9.0,             -- Divide by: 1.5,2.0,2.5,3.0,3.5,4.0,4.5,5.0,5.5,6.0,6.5,
                                              -- 7.0,7.5,8.0,9.0,10.0,11.0,12.0,13.0,14.0,15.0 or 16.0
        CLKFX_DIVIDE => 2,                     -- Can be any interger from 1 to 32
        CLKFX_MULTIPLY => 2,--3,               -- Can be any integer from 2 to 32
        CLKIN_DIVIDE_BY_2 => false,            -- TRUE/FALSE to enable CLKIN divide by two feature
      --CLKIN_PERIOD => 30.303,                -- Specify period of input clock in ns from 1.25 to 1000.00
        CLKIN_PERIOD => 10.0,                  -- Specify period of input clock in ns from 1.25 to 1000.00
        CLKOUT_PHASE_SHIFT => "NONE",          -- Specify phase shift mode of NONE or FIXED
        CLK_FEEDBACK => "1X",                  -- Specify clock feedback of NONE or 1X
        DCM_AUTOCALIBRATION => TRUE,           -- DCM calibrartion circuitry TRUE/FALSE
        DCM_PERFORMANCE_MODE => "MAX_SPEED",   -- Can be MAX_SPEED or MAX_RANGE
        DESKEW_ADJUST => "SYSTEM_SYNCHRONOUS", -- SOURCE_SYNCHRONOUS, SYSTEM_SYNCHRONOUS or an integer from 0 to 15
        DFS_FREQUENCY_MODE => "LOW",           -- LOW or HIGH frequency mode for frequency synthesis
        DLL_FREQUENCY_MODE => "LOW",           -- LOW, HIGH, or HIGH_SER frequency mode for DLL
        DUTY_CYCLE_CORRECTION => TRUE,         -- Duty cycle correction, TRUE or FALSE
        FACTORY_JF => X"F0F0",                 -- FACTORY JF Values Suggested to be set to X"F0F0" 
        PHASE_SHIFT => 0,                      -- Amount of fixed phase shift from -255 to 1023
        STARTUP_WAIT => false)                 -- Delay configuration DONE until DCM LOCK, TRUE/FALSE
    port map (
        CLK0 => clk0_buf,                      -- 0 degree DCM CLK ouptput
        CLK180 => open,                        -- 180 degree DCM CLK output
        CLK270 => open,                        -- 270 degree DCM CLK output
        CLK2X => open,                         -- 2X DCM CLK output
        CLK2X180 => open,                      -- 2X, 180 degree DCM CLK out
        CLK90 => open,                         -- 90 degree DCM CLK output
        CLKDV => open,                         -- Divided DCM CLK out (CLKDV_DIVIDE)
        CLKFX => clkfx_buf,                         -- DCM CLK synthesis out (M/D)
        CLKFX180 => open,                      -- 180 degree CLK synthesis out
        LOCKED => locked_sig,                  -- DCM LOCK status output
        CLKFB => clkfb_in,                     -- DCM clock feedback
        CLKIN => clkin_ibufg,                  -- Clock input (from IBUFG, BUFG or DCM)
        RST =>  '0'                            -- DCM asynchronous reset input
    );

    CLKDV_BUFG_inst : BUFG
    port map (
        I=>clkfx_buf,
        O=>clk_sig
    );
   
    CLKIN_BUFG_inst : BUFG
    port map (
        I=>CLK_100MHZ,
        O=>clkin_ibufg
    );
   
    CLK0_BUFG_inst : BUFG
    port map (
        I=>clk0_buf,
        O=>clkfb_in
    );

    counter_inst : simple_counter
    generic map ( 
        DATA_WIDTH => 8
    )
    port map ( 
        CLK => clk_sig,
        RST => rst_sig,
        CLR => counter_clr_sig,
        EN => counter_en_sig,
        DATA => counter_data_sig
    ); 

    delay_counter_inst : counter
    generic map ( 
        DATA_WIDTH => log2(SPEED*100000)
    )
    port map ( 
        CLK => clk_sig,
        RST => rst_sig,
        CLR => delay_counter_clr_sig,
        EN => delay_counter_en_sig,
        DATA => delay_counter_data_sig
    ); 

    switch_N_inst : switch
    generic map (
        FREQ_CLK => 100000000,   -- frekvence 100MHz
        LENGTH   => 500          -- delka stisku = 500 ms
    )
    port map (
        CLK => clk_sig,
        RST => rst_sig,
        SW  => GPIO_SW_N,
        PRESSED => pause_sig,
        BUSY => open
    ); 


    ila: chipscope_ila
    port map (
        CLK => clk_sig,
        TRIG0 => chipscope_trigger_sig,
        DATA => chipscope_data_sig,
        CONTROL => chip_controls
    );


    ila1: chipscope_ila1
    port map (
        CLK => clk_sig,
        TRIG0 => "00000000",--chipscope1_trigger_sig,
        DATA => "0000000000000000",--chipscope1_data_sig,
        CONTROL => chip1_controls
    );


    icon: chipscope_icon
    port map (
        CONTROL0 => chip_controls,
        CONTROL1 => chip1_controls
    );



    delay_finished_sig <= '1' when (delay_counter_data_sig >= conv_std_logic_vector(SPEED*100000, log2(SPEED*100000))) else '0';

    state_current : process(clk_sig,rst_sig)
    begin
        if (rst_sig='1') then
            state <= INIT;
        elsif rising_edge(clk_sig) then
            state <= nextstate;
        end if;
    end process;
     
    state_machine : process(state, delay_finished_sig, pause_sig)
    begin
        counter_clr_sig <= '0';
        counter_en_sig <= '0';
        delay_counter_clr_sig <= '0';
        delay_counter_en_sig <= '0';
        nextstate <= INIT;

        case state is
		        
        when INIT =>
            counter_clr_sig <= '1';
            delay_counter_clr_sig <= '1';		
            nextstate <= COUNT;  
				
        when COUNT =>
            if(pause_sig='0') then
                delay_counter_en_sig <= '1';
                
                if(delay_finished_sig='1') then
                    counter_en_sig <= '1';
                    delay_counter_clr_sig <= '1';
                end if;
                nextstate <= COUNT;
            else                
                delay_counter_clr_sig <= '1';
                nextstate <= PAUSE;
            end if;  

        when PAUSE =>
            if(pause_sig='1') then
                nextstate <= COUNT;
            else
                nextstate <= PAUSE;
            end if;
  
        when others => nextstate <= INIT;
				
        end case;
    end process;

    rst_sig <= GPIO_SW_C;

    GPIO_LED_0 <= counter_data_sig(0);
    GPIO_LED_1 <= counter_data_sig(1);
    GPIO_LED_2 <= counter_data_sig(2);
    GPIO_LED_3 <= counter_data_sig(3);
    GPIO_LED_4 <= counter_data_sig(4);
    GPIO_LED_5 <= counter_data_sig(5);
    GPIO_LED_6 <= counter_data_sig(6);
    GPIO_LED_7 <= counter_data_sig(7);	 
    
    HDR1_56 <= locked_sig;
    HDR1_60 <= clk_sig; 
    HDR1_64 <= CLK_100MHZ;


    chipscope_data_sig <= delay_counter_data_sig & delay_finished_sig & delay_counter_en_sig & delay_counter_clr_sig & pause_sig & counter_en_sig & counter_clr_sig & counter_data_sig & "00000" & "0000000000" & "0000000000" & "0000000000" & "0000000000" & "0000000000";
    chipscope_trigger_sig <= delay_finished_sig & delay_counter_en_sig & delay_counter_clr_sig & pause_sig & counter_en_sig & counter_clr_sig & "00";
end top_level_structural;

